
/*
# soexp: written by Brane Zdralo brane.zdralo@fe.uni-lj.si <3/12/2000>
# It adjusts the pagesize of an EPS or single page PS file to fit content
# and converts it into a PNG or encapsulated PDF file. For this it requires
# GhostScript6.0 or newer. Also it recognises Tango schematic and pcb
#	PS plot files and makes possible to generate color PNG or PDF files out of them.
#	The way soexp makes conversion of EPS files is the same as in program 
# epstopdf on which is based. The way soexp makes conversion of single page PS
# and Tango PS file is a little bit different.
# Epstopdf was written by Kong Hoon Lee, konghoon@hyowon.cc.pusan.ac.kr <4/1/1999>
# and changed by Juergen Bausa, bausa@lfpt.rwth-aachen.de <1/11/2000>
# Epstopdf is based on the perl script 'epstopdf' by Sebastian Rahtz on
# http://tug.org/applications/pdftex/epstopdf .
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_IN 10000

#ifndef GSEXEC
 #define GSEXEC "gswin32c"
#endif


struct colorproto{
	double red;
	double green;
	double blue;
};


/*default tango colours*/

struct colorproto coltable[]={
		{0.0,0.0,0.0},
		{1.0,0.0,0.0},
		{0.0,1.0,0.0},
		{0.0,0.0,1.0},
		{1.0,1.0,0.0},
		{0.0,1.0,1.0},
		{1.0,0.0,1.0},
		{0.5,0.0,0.0},
		{0.0,0.5,0.0},
		{0.0,0.0,0.5},
		{0.5,0.5,0.0},
		{0.0,0.5,0.5},
		{0.5,0.0,0.5},
		{0.25,0.0,0.0},
		{0.0,0.25,0.0},
		{0.0,0.0,0.25},
	};


static void usage(void)

{
   printf("soexp: written by Brane Zdralo brane.zdralo@fe.uni-lj.si <3/12/2000>\n");	
	 printf("It adjusts the pagesize of an EPS or single page PS file to fits content\n");
	 printf("and converts it into a PNG or encapsulated PDF file. For this it requires\n");
	 printf("GhostScript6.0 or newer. Also it recognises Tango schematic and pcb\n");
	 printf("PS plot files and makes possible to generate color PNG or PDF files from them\n");
	 printf("soexp is based on epstopdf:\n");  	 
   printf("written by Kong Hoon Lee, konghoon@hyowon.cc.pusan.ac.kr <4/1/1999>\n");
   printf("changed by Juergen Bausa, bausa@lfpt.rwth-aachen.de <1/11/2000>\n");
   printf("epstopdf is based on the perl script 'epstopdf' by Sebastian Rahtz on\n");
   printf("http://tug.org/applications/pdftex/epstopdf .\n");
   printf("Usage: soexp [options] filename-of-an-eps-file\n");
   printf("Options:\n");
   printf(" --help:                  print usage\n");
	 printf(" --initfile=<file>:       read settings from init file\n");
   printf(" --outfile=<file>:        write result to <file>\n");
   printf(" --outtype=<pdf/png/eps>: make final conversion             (default: pdf)\n");
   printf(" --nocompress:            for pdf don't use compression     (default: use)\n");
   printf(" --debug:                 debug informations                (default: nodebug)\n");
   printf(" --gsexec=<gs>:           use <gs> to invoke ghostscript    (default: %s)\n",GSEXEC);
   printf(" --notrim:                adjust bbox size to fits contents (default: trim)\n");
}


int main(int argc,char *argv[])

{
	
	/* Postscript strings */

	char psheaderstring[]="!PS-Adobe-";
	char pagesstring[]="%%Pages:";
	char pagestring[]="%%Page:";
	char bbstring[]="%%BoundingBox:";
	char creatorstring[]="%%Creator:";


   /* Windows PS file strings */

	char mystring[]="/mysetup [";
	char pagesizestring[]="dup /PageSize [";


	/* Tango strings */
	
	char oldfontstring[]="Helvetica";
	char graystring[]="setgray";
	char colorstring[]="setrgbcolor";
	char schstring[]="Schematic";
	char pcbstring[]="PCB";
	char mulstring[]="/m {";
	char srstring[]="/sr {";
	char prstring[]="/pr";

	FILE *in,*out;
	char *infile=NULL,*outfile=NULL,*initfile=NULL,*tempfile=NULL,*gsexec=NULL;
	char *copt,*ptr,*bbfile;
	char command[300],ch[MAX_IN],ttemp[MAX_IN],newfontstring[300],pngcol[10];
	double dbbox[4],mul,temp[6],border,pngxdimm;
	double rout,gout,bout,thick,thin,newthick,newthin,lwidth;
	int bbox[4],width,height,xoffset,yoffset,page,pages,soffset,sstart;
	int i,count,status;
	char BBCorrected,did_CR,debug,compress,dopng,dopdf,trim,eps,ps;
	char pr,sr,tgocol,sch,pcb,pngxfactor;	

	/* default parameters */

	status=0;
	debug=0;
	compress=1;
	dopdf=1;
	dopng=0;
	trim=1;
	bbfile="bbox.tmp";
	border=0;
	BBCorrected=0;
	pngxdimm=700;
	pngxfactor=0;

	page=0;
	pages=1;
	eps=0;
	ps=0;

	tgocol=1;
	sch=0;
	pcb=0;
	thick=0;
	thin=0;
	newthick=0.8;
	newthin=0.8;
	lwidth=0;
	sr=0;
	pr=0;
	strcpy(newfontstring,"Helvetica");
	strcpy(pngcol,"png16m");


	/* Look up for init file and process it */
	
	for(i=1;i<argc;i++){
		if(!strncmp(argv[i],"--initfile=",strlen("--initfile="))){
			initfile=malloc(strlen(argv[i])-strlen("--initfile=")+1);
			strcpy(initfile,argv[i]+strlen("--initfile="));
			if(initfile != NULL){
				if((in=fopen(initfile, "r")) != NULL){
					if(debug) printf("init file = >>%s<<.\n", initfile); 
					else printf("Reading init file...\n", initfile);
					while (fgets(ch,MAX_IN,in) != NULL ){	
						if(ch[0]=='#' || ch[0] == '\n' ) continue;
						if(strncmp(ch,"pngcol",strlen("pngcol")) == 0){
							sscanf(ch+strlen("pngcol"),"%s", pngcol);
							continue;
							}
						if(strncmp(ch,"gsexec",strlen("gsexec")) == 0){
							sscanf(ch+strlen("gsexec"),"%s", ttemp);
							gsexec=malloc(strlen(ttemp)+1);
							strcpy(gsexec,ttemp);
							continue;
						}

						if(strncmp(ch,"outtype",strlen("outtype")) == 0){
							sscanf(ch+strlen("outtype"),"%s", ttemp);
							if(!strcmp("png",ttemp)){
								dopdf=0;
								dopng=1;
								continue;
							}
							if(!strcmp("eps",ttemp)){
								dopdf=0;
								continue;
							}
							if(!strcmp("pdf",ttemp)) continue;
							printf("Unknown output format %s!\n", ttemp);
							printf("Using default output format pdf.\n");							
							continue;
						}						
						if(strncmp(ch,"notrim",strlen("notrim")) == 0){
							trim=0;
							continue;
						}						
						if(strncmp(ch,"trim",strlen("trim")) == 0){
							sscanf(ch+strlen("trim"),"%lf", &border);
							continue;
						}						
						if(strncmp(ch,"nocompress",strlen("nocompress")) == 0){
							compress=0;					
							continue;
						}		
						if(strncmp(ch,"debug",strlen("debug")) == 0){
							debug=1;					
							continue;
						}
						if(strncmp(ch,"notgocol",strlen("notgocol")) == 0){
							tgocol=0;
							continue;
						}	
						if(strncmp(ch,"color",strlen("color")) == 0){
							sscanf(ch+strlen("color"),"%d %lf %lf %lf", &i,&rout,&gout,&bout);
							if(rout > 1) rout=1;
							if(gout > 1) gout=1;				
							if(bout > 1) bout=1;
							coltable[i-1].red=rout;gout=coltable[i-1].green=gout;coltable[i-1].blue=bout;
							continue;
						}
						if(strncmp(ch,"font",strlen("font")) == 0){
							sscanf(ch+strlen("font"),"%s", newfontstring);
							continue;
						}
						if(strncmp(ch,"thin",strlen("thin")) == 0){
							sscanf(ch+strlen("thin"),"%lf", &newthin);
							continue;
						}
						if(strncmp(ch,"thick",strlen("thick")) == 0){
							sscanf(ch+strlen("thick"),"%lf", &newthick);
							continue;
						}
						if(strncmp(ch,"pngxdimm",strlen("pngxdimm")) == 0){
							sscanf(ch+strlen("pngxdimm"),"%lf", &pngxdimm);
							pngxfactor=0;
							continue;
						}						
						if(strncmp(ch,"pngxfactor",strlen("pngxfactor")) == 0){
							sscanf(ch+strlen("pngxfactor"),"%lf", &pngxdimm);
							pngxfactor=1;
							continue;
						}						
						ch[strlen(ch)-1]='\0';
						printf("Unknown parameter >>%s<<\n", ch);											
					}
					fclose(in);
					free(initfile);
					break;
				}
			else printf("Initfile not found, continuing with defaults.\n");
			break;
			}
		}
	}
	

	/* Process command line options */
	
	if(!debug)printf("Processing command line...\n");
	for(i=1;i<argc;i++){
		if(debug)printf("arg[%d]=>>%s<<\n",i,argv[i]);


		if(!strcmp(argv[i],"--help") || !strcmp(argv[i],"-h")){
			usage();
			status=0;
			goto EXIT;
		}


		if(!strcmp(argv[i],"--nocompress") || !strcmp(argv[i],"-nc")){
			compress=0;
			continue;
		}


		if(!strcmp(argv[i],"--debug") || !strcmp(argv[i],"-d")){
			debug=1;
			continue;
		}


		if(!strcmp(argv[i],"--notrim") || !strcmp(argv[i],"-nt")){
			trim=0;
			continue;
		}


		if(!strncmp(argv[i],"--outfile=",strlen("--outfile="))){
			outfile=malloc(strlen(argv[i])-strlen("--outfile=")+1);
			strcpy(outfile, argv[i]+strlen("--outfile="));
			continue;
		}

		if(!strcmp(argv[i],"-o") && i+1<argc){
			outfile=malloc(strlen(argv[i+1])+1);
			strcpy(outfile, argv[i+1]);
			i++;
			continue;
		}


		if(!strncmp(argv[i],"--initfile=",strlen("--initfile="))) continue;


		if(!strncmp(argv[i],"--outtype=",strlen("--outtype="))){
			if(!strcmp("pdf",argv[i]+strlen("--outtype="))){
				dopdf=1;
				dopng=0;
				continue;
			}
			if(!strcmp("png",argv[i]+strlen("--outtype="))){
				dopdf=0;
				dopng=1;
				continue;
			}
			if(!strcmp("eps",argv[i]+strlen("--outtype="))){
				dopdf=0;
				dopng=0;
				continue;
			}
			printf("Unknown output file type!");
			printf("Valid output types are: pdf, png or eps.");
			status=1;
			goto EXIT;
		}
	

		if(!strncmp(argv[i],"--gsexec=",strlen("--gsexec="))){
			if(gsexec != NULL) free(gsexec); 
			gsexec=malloc(strlen(argv[i])-strlen("--gsexec=")+1);
			strcpy(gsexec, argv[i]+strlen("--gsexec="));
			continue;
		}


		if(argv[i][0]!='-'){
			if(infile) printf("Can process only one input file\n");
			else       infile=argv[i];
			continue;
		}


		usage();
		printf("\nUnknown option >>%s<< !\n",argv[i]);
		goto EXIT;
	}


	if(!infile){
		usage();
		printf("\nNo input file specified!\n");
		status=1;
		goto EXIT;
	}


	if((in=fopen(infile,"r")) == NULL){
		usage();
		printf("\n%s: File not found!\n",infile);
		status=1;
		goto EXIT;
	}

	fclose(in);

	if(gsexec == NULL){
		gsexec=malloc(strlen(GSEXEC)+1);
		strcpy(gsexec,GSEXEC);
	}


	/* find a temporary filename that does not exist yet */

	tempfile=malloc(strlen(infile)+8);
	count=0;
	do{
		if(count>99){
			printf("No temporary file available!\n\n");
			status=1;
			goto EXIT;
		}
		sprintf(tempfile,"%s.%02d.tmp",infile,count);
		if(debug) printf("Checking temporary filename %s .\n",tempfile);
		out=fopen(tempfile,"r");
		if(out) fclose(out);
		count++;
	}while(out);

	 
	/* test input file for format, creator, etc.. */

	in=fopen(infile, "r");
		
	while ((fgets(ch,MAX_IN,in)) !=NULL){

		/* test input format - PS or EPS */
	
		if(strstr(ch,psheaderstring) != NULL ){
			ps=1;
			if(strstr(ch,"EPSF") != NULL){
				ps=0;
				eps=1;
			}
			continue;
		}


		/* test if only one page - %%Pages 1 */

		if(strncmp(ch,pagesstring,strlen(pagesstring)) == 0 ){
			sscanf(ch+strlen(pagesstring),"%d",&pages);
			if(pages > 1){
				fclose(in);
				printf("You can convert only single page documents!");
				status=1;
				goto EXIT;
			}
		}


		/* test if creator is Tango schematic or pcb */
		
		if (strncmp(ch, creatorstring,strlen(creatorstring)) == 0 ){
			if(strstr(ch,pcbstring) != NULL) pcb=1;
			if(strstr(ch,schstring) != NULL) sch=1;
			if((pcb || sch) && eps){
				fclose(in);
				printf("In Tango you should use PS only!");
				status=1;
				goto EXIT;
			}
			continue;
		}
			
			
		/* for Tango schematic find thick and thin line width */

		if(sch){
			if(strstr(ch," pt") != NULL){
				sscanf(strstr(ch," pt")-3,"%lf", &lwidth);
				if(lwidth != thick){
					if(lwidth > thick){	
						thin=thick;
						thick=lwidth;
					}
					else thin=lwidth;
					continue;
				}
			}
		}
	}
	
	fclose(in);
	

	/* test if PS or EPS file */

	if(!ps && !eps){
		printf("Input file is not in PS or EPS format!");
		status=1;
		goto EXIT;
	}

	
	/* processing trim */

	if(trim){
		sprintf(command,"%s -dNOPAUSE -dBATCH -sDEVICE=bbox %s > %s",gsexec,infile,bbfile);
		if(!debug) printf("Calculating Trimmed Bounding Box ...\n");
		else printf("%s\n",command);
		status=system(command);
		if(status == -1) goto EXIT;
		if((in=fopen(bbfile,"r")) == NULL){
			printf("Can't find %s!\n", bbfile);
			status=1;
			goto EXIT;
		}
		while ((fgets(ch,MAX_IN,in)) !=NULL){
			if (strncmp(ch,bbstring,strlen(bbstring)) == 0 ) {
				sscanf(ch+strlen(bbstring),"%lf %lf %lf %lf",&dbbox[0],&dbbox[1],&dbbox[2],&dbbox[3]);
				if(border){
					border=72*border;
					dbbox[0]=dbbox[0]-border;
					dbbox[1]=dbbox[1]-border;
					dbbox[2]=dbbox[2]+border;
					dbbox[3]=dbbox[3]+border;
				}
				for(i=0;i<4;i++) bbox[i]=dbbox[i];
				break;
			}
		}
		fclose(in);
		if(debug)printf("Trimed BoundingBox is: %.0lf %.0lf %.0lf %.0lf.\n",dbbox[0],dbbox[1],dbbox[2],dbbox[3]);
		else remove(bbfile);
	}

	
	/* generating output filename */
	
	if(!outfile){
		outfile=malloc(strlen(infile)+7);
		strcpy(outfile,infile);
		ptr=strrchr(outfile,'.');
		if(ptr) *ptr='\0';
		if(dopdf)	strcat(outfile,".pdf");
		else	if(dopng) strcat(outfile,".png");
					else      strcat(outfile,".2.eps");
	}

	
	/* process Windows PS print file */

	if(ps && !(pcb || sch)){
		in=fopen(infile, "r");
		out=fopen(tempfile,"w");

		while ((fgets(ch,MAX_IN,in)) !=NULL){


			/* change bounding box */

			if (strncmp(ch,bbstring,strlen(bbstring)) == 0 ){
				if(!trim)   sscanf(ch+strlen(bbstring),"%lf %lf %lf %lf",&dbbox[0],&dbbox[1],&dbbox[2],&dbbox[3]);
					width   = dbbox[2]-dbbox[0];
					height  = dbbox[3]-dbbox[1];
					fprintf(out,"%s 0 0 %d %d\n", bbstring,width,height);
					BBCorrected=3;
					continue;
			}

			/* change PageSize */

			if(strstr(ch,pagesizestring) != NULL){
				fprintf(out,"    %s%d %d] put\n", pagesizestring,width,height);
				BBCorrected--;
				continue;
			}

			/* change coordinate system - /mysetup */
			
			if(strncmp(ch,mystring,strlen(mystring)) == 0){
				sscanf(ch+strlen(mystring),"%lf %lf %lf %lf %lf %lf", &temp[0],&temp[1],&temp[2],&temp[3],&temp[4],&temp[5]);
				temp[4]=temp[4]-dbbox[0];
				temp[5]=temp[5]-dbbox[1];
				fprintf(out,"%s %.3lf %.3lf %.3lf %.3lf %.3lf %.3lf ] |\n", mystring,temp[0],temp[1],temp[2],temp[3],temp[4],temp[5]);
				BBCorrected--;
				continue;
			}

			/* copy unchanged lines */
			
			fputs(ch,out);

		}
		fclose(in);
		fclose(out);
		if(BBCorrected != 1)BBCorrected=0;
	}
	 
	 
	/* process Tango file */
	
	if(pcb || sch){

		if(sch){
			newthick=thick*newthick;
			newthin=thin*newthin;
			if(debug){
				printf("Thick is %.0lf, thin is %.0lf\n", thick,thin);
				printf("New thick is %0.1lf, new thin is %0.1lf\n", newthick,newthin);
			}
		}

		in=fopen(infile, "r");
		out=fopen(tempfile,"w");
	
		while ((fgets(ch,MAX_IN,in)) !=NULL){


			/* change tango font */

			if(strstr(ch, oldfontstring) != NULL){			
				sstart=strstr(ch, oldfontstring)-ch;
				soffset=strlen(newfontstring)-strlen(oldfontstring);
				for(i=0;ch[i] != '\0' ;i++){
					if(i < sstart)ttemp[i]=ch[i];
					else if(i+soffset >= sstart)ttemp[i+soffset]=ch[i];
				}
				for(i=0;newfontstring[i] != '\0';i++)ttemp[i+sstart]=newfontstring[i];
				fprintf(out,"%s", ttemp);
				continue;
			}


			/* change bounding box */

			if (strncmp(ch,bbstring,strlen(bbstring)) == 0 ){
				if(!trim){
					sscanf(ch+strlen(bbstring),"%lf %lf %lf %lf",&dbbox[0],&dbbox[1],&dbbox[2],&dbbox[3]);
				}
				width   = dbbox[2]-dbbox[0];
				height  = dbbox[3]-dbbox[1];
				fprintf(out,"%s 0 0 %d %d\n", bbstring,width,height);
				fprintf(out,"<< /PageSize [%d %d] >> setpagedevice \n",width,height);
				continue;
			}


			/* change coordinate system */ 
		
			if(strncmp(ch,mulstring,strlen(mulstring)) == 0){
				sscanf(ch+strlen(mulstring),"%lf",&mul);
			}
			if(strncmp(ch,pagestring,strlen(pagestring)) == 0) page=1;
			if(page && trim){
				if(strstr(ch, "translate") != NULL){
					sscanf(ch,"%lf %s %lf",&dbbox[2],ttemp,&dbbox[3]);
					xoffset =dbbox[2]-dbbox[0]/mul;
					yoffset =dbbox[3]-dbbox[1]/mul;	
					fprintf(out,"%d m %d m translate\n", xoffset, yoffset);
					continue;
				}
			}

			/* repairing square pad color - removing 1 graylevel from /sr definition */
			
			if(pcb){
				if(strstr(ch, srstring) != NULL) sr=1;
				if(sr && strstr(ch, graystring)){
					sr=0;
					continue;
				}
			}

			/* making colors from gray levels */

			if(page){
				if(tgocol){
					if(strstr(ch,graystring) != NULL){
						sscanf(ch+2,"%d", &i);
						i=i/5;
						rout=coltable[i].red;gout=coltable[i].green;bout=coltable[i].blue;
						fprintf(out,"%.2lf %.2lf %.2lf %s\n", rout,gout,bout,colorstring);
						continue;
					}
				}
			}

			/* changing line width */

			if(sch){
				if(strstr(ch, prstring) != NULL) pr=1;
				if(pr){
					if(strstr(ch,"a5 3 eq")){
						fprintf(out,"          {a5 3 eq {%.1lf} {%.1lf} ifelse "
							         "m setlinewidth stroke} ifelse\n", newthick, newthin);
						pr=0;
						continue;
					}
				}

				if(strstr(ch," pr") != NULL){
					sscanf(ch,"%lf %lf %lf %lf", &temp[0],&temp[1],&temp[2],&temp[3]);
					fprintf(out,"%.0lf %.0lf %.0lf %.0lf 4 pr\n", temp[0],temp[1],temp[2],temp[3]);
					continue;
				}

				if(strstr(ch," pt") != NULL){
					sscanf(ch,"%lf %lf %lf %lf %lf", &temp[0],&temp[1],&temp[2],&temp[3],&lwidth);
					if(lwidth == thick)lwidth = newthick;
					else lwidth = newthin;
					fprintf(out,"%.0lf %.0lf %.0lf %.0lf %.1lf pt\n", temp[0],temp[1],temp[2],temp[3],lwidth);
					continue;
				}

				if(strstr(ch," pa") != NULL){
					sscanf(ch,"%lf %lf %lf %lf %lf %lf", &temp[0],&temp[1],&lwidth,&temp[2],&temp[3],&temp[4]);
					if(lwidth == thick)lwidth = newthick;
					else lwidth = newthin;
					fprintf(out,"%.0lf %.0lf %.1lf %.0lf %.0lf %.0lf pa\n", temp[0],temp[1],lwidth,temp[2],temp[3],temp[4]);
					continue;
				}
		
			}

			/* copying unchanged line */

			 fputs(ch,out);

		}
		fclose(in);
		fclose(out);
		BBCorrected=1;
	}
		
	
	if(!BBCorrected){
		printf("Converting %s to %s ..... ",infile,outfile);
		if(debug) printf("\nAdding correct pagesize to EPS ...\n");
		in=fopen(infile, "r");
		out=fopen(tempfile,"w");
		while ((fgets(ch,MAX_IN,in)) !=NULL){
			if (strncmp(ch,bbstring,strlen(bbstring)) == 0 ){
				BBCorrected=1;
				if(!trim){
					sscanf(ch+strlen(bbstring),"%lf %lf %lf %lf",&dbbox[0],&dbbox[1],&dbbox[2],&dbbox[3]);
					for(i=0;i<4;i++) bbox[i]=dbbox[i];
				}
				if(debug) printf("%s %d %d %d %d\n", bbstring,bbox[0],bbox[1],bbox[2],bbox[3]);
				width   = bbox[2]-bbox[0];
				height  = bbox[3]-bbox[1];
				xoffset =-bbox[0];
				yoffset =-bbox[1];
				fprintf(out,"%s %d %d %d %d\n", bbstring,0,0,width,height);
				fprintf(out,"<< /PageSize [%d %d] >> setpagedevice \n",width,height);
				fprintf(out,"gsave %d %d translate\n",xoffset,yoffset);
				did_CR=1;
			}
			else{
				fputs(ch,out);
				if(strpbrk(ch, "\n")) did_CR=1;
				else                  did_CR=0;
			}
		}
		if(BBCorrected){
			if(!did_CR) fprintf(out,"\ngrestore\n");
			else        fprintf(out,  "grestore\n");
		}
		fclose(in);
		fclose(out);
	}

	
	if((out=fopen(outfile,"w")) == NULL){
		printf("Cannot open output file!\n");
		printf("You cannot simultaneously preview picture in Adobe Acrobat\n");
		printf("and generate new picture with the same name!\n");
		remove(tempfile);
		status=1;
		goto EXIT;
	}
	fclose(out);


	if(dopdf || dopng){
		if(dopdf){
			if(!compress) copt="-dUseFlateCompression=false ";
			else					copt="-dUseFlateCompression=true ";
			sprintf(command,"%s -q -dNOPAUSE -dBATCH -dCompatibilityLevel=1.2 %s"
							        "-dColorImageResolution=300 -sDEVICE=pdfwrite -sOutputFile=%s  -c save pop -f %s",
											gsexec,copt,outfile,tempfile);
		}
		else{
			if(dopng){
				if(!pngxfactor)	pngxdimm=72*pngxdimm/width;
				sprintf(command,"%s -q -dNOPAUSE -dBATCH"
												" -sDEVICE=%s -r%.0lf -sOutputFile=%s %s",
												gsexec,pngcol,pngxdimm,outfile,tempfile);
			}
		}		
		if(debug) printf("%s\n", command);
		else			printf("Running ghostscript ...\n");
		status=system(command);
	}	
	else{
		if(out=fopen(outfile, "r")){
			fclose(out);
			printf("File %s will be overwritten.\n", outfile);
			remove(outfile);
		}
		rename(tempfile,outfile);
	}
	if(debug) printf("Leaving temporary files: %s and %s.\n", bbfile,tempfile);
	else remove(tempfile);
	printf("Done!\n");	

EXIT:
	if(tempfile != NULL) free(tempfile);
	if(outfile != NULL) free(outfile);
	if(gsexec != NULL) free(gsexec); 
	return status;	
}

